// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef IPV6_HEADER_H
#define IPV6_HEADER_H

#include <stdint.h>

#include <boost/asio/ip/address_v6.hpp>

#include "host/messagepayload.h"

//-----------------------------------------------------------------------------
// Ipv6Header
//-----------------------------------------------------------------------------

/**
 * @brief This class represents the IP version 6 Packet Header.
 *
 * The IPv6 header format is:
 *
 * @code
 * 0      3 4            11 12   15 16          23 24            31
 * +-------+---------------+--------------------------------------+      ---
 * |       |               |                                      |       ^
 * |version| traffic class |              flow label              |       |
 * |  (6)  |               |                                      |       |
 * +-------+---------------+-------+--------------+---------------+       |
 * |                               |              |               |       |
 * |    payload length in bytes    | next header  |   hop limit   |       |
 * |                               |              |               |       |
 * +---------------+---------------+--------------+---------------+       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                      source IPv6 address                     |       |
 * |                                                              |   40 bytes
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * +--------------------------------------------------------------+       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                   destination IPv6 address                   |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       |
 * |                                                              |       v
 * +--------------------------------------------------------------+      ---
 * @endcode
 */
class Ipv6Header
{
public:
    Ipv6Header();

    uint8_t get_version() const;
    uint8_t get_differentiated_services() const;
    uint32_t get_flow_label() const;

    uint16_t get_payload_length() const;
    uint8_t get_next_header() const;
    uint8_t get_hop_limit() const;

    boost::asio::ip::address_v6 get_source_address() const;
    boost::asio::ip::address_v6 get_destination_address() const;

    friend std::istream &operator>>(
            std::istream &is,
            Ipv6Header &header
    );

private:
    MessagePayload Payload;

};

#endif // IPV6_HEADER_H
